% Create_Simulation_Matrix.m
% Jason Wu 
% 01/26/15

% Edited from:
% - createSimulationMatrix_openSHA_wrapper by Mahalia Miller
% - distanceFromLongLat by Mahalia Miller
% - createSimulationMatrix_openSHA_sub by Mahalia Miller
%
% Creates matrix where first column is Sa  in site ID 1,
% second is Aa in site ID 2, ...site ID n where n is the number of sites.
% Each row corresponds to a different earthquake scenario. Refer to the
% metadata output from this code to see details regarding each earthquake
% scenario. 
% One simulation matrix is created for each combination of GMPE and IM
%
% PREPROCESSING
%
% Before this code can be run, we must obtain results from OpenSHA.
% 1) Edit and run AWSS_OpenSHA_Input_Writer.m to write the input file.
% 2) To go the start menu and search cmd.exe
% 3) Find the folder that the OpenSHA information is located 
%    Note: use "cd .." to go up a level and "cd <folder name>" to connect
%    to the folder
% 4) Type the following on the command line:
% java -jar -Xmx500M IM_EventSetCalc_v3_0_ASCII.jar <name of the inputfile> <name of output folder>
%
% This should create at least 4 outputfiles we will use in this code:
% 1) Text file containing mean/std dev intensity values per site. File name
% is based on the attenuation function used and the intensity measure
% 2) 2 text files containing ground motion sources (e.g. rupture and distance)
% File name is always "rup_dist_info.txt" and "rup_dist_jb_info.txt"
% 3) Text file containing meta data. File name is always
% "src_rup_metadata.txt"
%
% INPUTS
%
% If input structures are NOT specified, the following parameters must be
% defined by the user in the INPUTS section below:
%
% File Organization.
% Masterfolder : String : Indicates the path/directory of the folder
%       containing the files related to this project (e.g. site list)
% SimulationFolder : String : Indicates the path/directory of the folder to
%       contain the results of the ground motion simulation
% OpenSHAOutputFolder : String : Indicates the path/directory of the folder
%       containing the results of the OpenSHA calculations
% 
% Other Inputs. 
% nSimBatches : Scalar : Number of desired batches of simulations, where
%       each batch includes a ground motion simulation for each site and
%       each earthquake scenario (i.e. source/rupture case defined by OpenSHA).
% 
% Options.
% RandomFieldFlag : Integer : This indicates the random field simulation
%       method to be used.
%       1 = Covariance Method
%       2 = Spectral Method
%       3 = Sequential Method
%       4 = Turning Bands Method
%       5 = Circulant Embedding Method
% OverwriteFlag : Logical : This indicates whether previously calculated
%       results are to be overwritten.
%       0 = do not overwrite files
%       1 = overwrite files
% SuppressMessagesFlag : Logical : This indicates whether to suppress
%       messages regarding simulation progress
%       0 = do not suppress messages
%       1 = messages are suppressed
% MedFlag : Logical : This indicates whether to output median values instead
%       of simulating correlated ground motion values
%       0 = do not output median values--simulate correlated ground motion
%           values
%       1 = output median values
%
% Random Field Simulation Options.
% GridSpacing : Scalar : This indicates the spacing between sites in
%       kilometers. Used for the spectral and circulant embedding methods
% Spectral_scale : Integer : Positive integer indicating the number of sinusoids
%       used for simulation. Increased scale may improve accuracy, but also may
%       take more time.
%       scale = 1 = number of sinusoids equal to the number of sites in
%           each row or column in the site grid (the larger of the two).
%       scale = 2 = twice the number of sinusoids as scale = 1
%       scale = x = x times the number of sinusoids as scale = 
% Sequential_useCoarse : Logical : This indicates whether to precompute a
%       coarse grid for the sequential method.
%       0 = do not precompute a coarse grid
%       1 = precompute a coarse grid
% Sequential_coarseSize : Integer : This indicates the number of sites in
%       each column and row of the coarse grid
% Sequential_maxDist : Scalar : This indicates the maximum distance to
%       search for neighboring sites for the simulation to condition on
% Sequential_maxScreen : Integer : This indicates the maximum number of
%       sites to condition on during simulation
% OUTPUTS
%
% simulationMatrix : Matrix : m x n matrix
%       This should be in the form of the following:
%       [SA(site1) SA(site2) ... SA(siteI)]
% GroundMotionInfo : Structure : This structure contains the following
%       information regarding the ground motion:
% GroundMotionInfo.mu : Matrix : m x n matrix describing the median/mean
%       ground motion values for each site and simulation case from
%       OpenSHA. There are m simulation cases and n sites.
% GroundMotionInfo.sigma : Matrix : m x n matrix describing the standard
%       deviation of the intraevent residual from OpenSHA. There are m
%       simulation cases and n sites.
% GroundMotionInfo.tau : Matrix : m x n matrix describing the standard
%       deviation of the interevent residual from OpenSHA. There are m
%       simulation cases and n sites.
%
% OTHER VARIABLES
% 
% siteList : Matrix : n x 3 matrix listing the latitude, longitude, and
%       V_s30 values for each site for n sites. This is extracted from the
%       file defined by siteListFilename, which should be in the following
%       format:
%       [latitude longitude V_s30]
% metaData : Matrix : m x 4 matrix listing the source ID, rupture ID,
%       annualized rate, and magnitude for each simulation case, of which
%       there are a total of m. This is extracted from the file defined by
%       metaDataFilename, which should be in the following format:
%       [sourceID ruptureID annualizedRate magnitude ...
%       <name of the source/rupture case>]
%       Note that the <name> may consist of multiple words, and since
%       textscan considers white space as delimiters, this requires
%       multiple "%s" in the function call. Due to both this complexity and
%       the fact that this is a string, this is omitted from the generated
%       simulation matrix and instead saved in a separate variable named
%       metaDataDescription.
% metaDataDescription: Cell : m x 1 cell listing the name of the
%       source/rupture case for each simulation case. This is extracted
%       from the file defined by metaDataFilename, as described above.
% intensity : Matrix : m x (n*3+2) matrix storing the sourceID, ruptureID,
%       mean, total standard deviation, and inter-event standard deviation
%       values of the ground motion for each site (for n sites) and for
%       each source/rupture case (m cases). This is extracted from the file
%       defined by intensityFilename, which should be in the following
%       format:
%       [sourceID ruptureID mu(site 1) total_std_dev(site 1) ... 
%       inter-event_std_dev(site 1) ---> mu(site i) total_std_dev(site i) ... 
%       inter-event_std_dev(site i)]

function errFlag = Create_Simulation_Matrix(Files,Input,Options)

%% USER DEFINED INPUTS: (if input structures are not specified)
if nargin == 0

Identifier = 'Example';

BaseFolder = 'C:\Jason Files\Baker Research 13\';
MasterFolder = [BaseFolder 'Masterfolder_' Identifier '\'];
SimulationFolder = [MasterFolder 'Simulations\'];
OpenSHAOutputFolder = [BaseFolder 'OpenSHA\Output_' Identifier '\'];

nSimBatches = 1;
RandomFieldFlag = 1;
OverwriteFlag = 0;
SuppressMessagesFlag = 0;
MedFlag = 0;
GMseed = 123;

% **Random Field Simulation Options**
GridSpacing = 0.5;

% Spectral Formulation
Spectral_scale = 1;

% Sequential Formulation
Sequential_useCoarse = 1;
Sequential_coarseSize = 5;
Sequential_maxDist = 30;
Sequential_maxScreen = 30;

% Turning Bands Formulation
TurningBands_M = 100;
TurningBands_nLines = 16;
TurningBands_dels = 0.5;

%% Preparations
SpectralParams.scale = Spectral_scale;
SpectralParams.spacing = GridSpacing;
Input.SpectralParams = SpectralParams;

SequentialParams.useCoarse = Sequential_useCoarse;
SequentialParams.coarseSize = Sequential_coarseSize;
SequentialParams.maxDist = Sequential_maxDist;
SequentialParams.maxScreen = Sequential_maxScreen;
Input.SequentialParams = SequentialParams;

TurningBandsParams.M = TurningBands_M;
TurningBandsParams.nLines = TurningBands_nLines;
TurningBandsParams.dels = TurningBands_dels;
Input.TurningBandsParams = TurningBandsParams;

CircEmbParams.spacing = GridSpacing;
Input.CircEmbParams = CircEmbParams;

Options.RandomFieldFlag = RandomFieldFlag;
Options.OverwriteFlags = [1 OverwriteFlag];
Options.SuppressMessagesFlag = SuppressMessagesFlag;
Options.MedFlag = MedFlag;

else
%% Variable Assignment (if input structures are specified)

MasterFolder = Files.MasterFolder;
SimulationFolder = Files.SimulationFolder;
OpenSHAOutputFolderName = Files.OpenSHAOutputFolderName;
OpenSHAFolder = Files.OpenSHAFolder;
OpenSHAOutputFolder = [OpenSHAFolder OpenSHAOutputFolderName];

nSimBatches = Input.nSimBatches;
GMseed = Input.GMseed;

OverwriteFlag = Options.OverwriteFlags(2);
SuppressMessagesFlag = Options.SuppressMessagesFlag;
MedFlag = Options.MedFlag;

end

%% Additional Initialization

errFlag = 0;
metaDataFilename = 'src_rup_metadata.txt';
simListFilepath = [MasterFolder 'simData.mat'];
siteListFilepath = [MasterFolder 'SiteList.txt'];
groundMotionInfoFolder = [MasterFolder 'GroundMotionInfo\'];

if ~exist(simListFilepath, 'file') || OverwriteFlag == 1
    simList = cell(0);
else
    load(simListFilepath)
end

if ~exist(SimulationFolder,'dir') || OverwriteFlag == 1
    [~] = rmdir(SimulationFolder,'s');
    mkdir(SimulationFolder);
end

if ~exist(groundMotionInfoFolder,'dir') || OverwriteFlag == 1
    [~] = rmdir(groundMotionInfoFolder,'s');
    mkdir(groundMotionInfoFolder);
end

if SuppressMessagesFlag == 0
    disp('Creating Simulation Matrices')
end


%% File Reading

metaDataFilepath = [OpenSHAOutputFolder metaDataFilename];
if ~exist(metaDataFilepath,'file')
    disp('**ERROR** Metadata file output from OpenSHA not found.')
    disp('Possible failure in running OpenSHA. Check ground motion parameter inputs.')
    errFlag = 1;
    return
end

% Obtain the number of sites
[siteList, nSites] = extractSiteList(siteListFilepath);
if SuppressMessagesFlag == 0
    disp('Site List successfully loaded')
end

% Obtain the meta data
[metaData, metaDataDescription, numScenarios] = extractMetaData(metaDataFilepath);
if SuppressMessagesFlag == 0
    disp('Meta Data successfully loaded')
end
 
%% Create simulation matrices with correlated epsilons

FileList = dir(OpenSHAOutputFolder);
FileList = FileList(3:end); % Take out '.' and '..' folders
RemoveInd = [];

% Take out the files that are not ground motion intensity files
for i = 1:length(FileList)
    if strcmp(FileList(i).name,'rup_dist_info.txt');
        RemoveInd = [RemoveInd; i];
    elseif strcmp(FileList(i).name,'rup_dist_jb_info.txt');
        RemoveInd = [RemoveInd; i];
    elseif strcmp(FileList(i).name,'src_rup_metadata.txt');
        RemoveInd = [RemoveInd; i];
    end
end
 
FileList(RemoveInd) = [];

if isempty(FileList)
    disp('**ERROR** No OpenSHA output files found. Problems could be the following:')
    disp('1) Internect connectivity')
    disp('2) Problems with the communication with OpenSHA')
    disp('3) Errors in the OpenSHA input file')
    errFlag = 1;
    return
end

% For each ground motion intensity and attenuation function, create the
% simulation matrices

rng(GMseed);
for i = 1:length(FileList)
    intensityFilename = FileList(i).name;
    resultsFilename = intensityFilename(1:end-4);
    intensityFilepath = [OpenSHAOutputFolder intensityFilename];
    groundMotionInfoFilepath = [groundMotionInfoFolder resultsFilename '_GroundMotionInfo.mat'];
    
    intensity = extractIntensity(intensityFilepath, nSites);
    if SuppressMessagesFlag == 0
        disp(['Intensity File: ' resultsFilename ' successfully loaded'])
    end
    
    % This flag indicates whether the ground motion is stored as a logarithm or
    % its true value. Currently, it appears that PGA, PGV and SA is stored 
    % are stored as medians and logrithmic dispersions.
    LogFlag = 1;
    
    % If the file contains PGV, then perform epsilon calculations assuming a
    % period of 1 and change the LogFlag to 1
    if strcmp(intensityFilename(end-6:end-4),'PGV')
        period = 1;
        % If the file contains PGA, then perform epsilon calculations with a period
        % of 0 and change the LogFlag to 1
    elseif strcmp(intensityFilename(end-6:end-4),'PGA')
        period = 0;
        % If the file contains SA at some period, then perform epsilon calculations
        % with a period equal to that period.
    else
        period = intensityFilename(end-6:end-4);
        period = str2double(period);
    end
    
    % Addition inputs for calling random field simulation functions
    MoreInput.period = period;
    MoreInput.numScenarios = numScenarios;
    MoreInput.nSites = nSites;
    MoreInput.siteList = siteList;
    
    savedGMInfo = 0;
    
    for j = 1:nSimBatches
        
        currResultFilename = [resultsFilename '_Sim' num2str(j) '.mat'];
        currSavepath = [SimulationFolder currResultFilename];
        if exist(currSavepath,'file') && OverwriteFlag == 0
            continue
        end
        
        if MedFlag == 0
            epsilons = getEpsilons(Files, Input, Options, MoreInput); % calculate epsilon realizations
            [simulationMatrix, GroundMotionInfo, errFlag] = calculateGroundMotion(metaData,intensity,epsilons,LogFlag,MedFlag);
        else
            numSites = size(siteList,1);
            [simulationMatrix, GroundMotionInfo, errFlag] = calculateGroundMotion(metaData,intensity,zeros(numScenarios,numSites),LogFlag,MedFlag);
        end
        save(currSavepath , 'simulationMatrix','epsilons'); % Save the aggregate matrix
        if ~exist(groundMotionInfoFilepath,'file') && savedGMInfo == 0
            save(groundMotionInfoFilepath, 'GroundMotionInfo'); % Save the info regarding the EQ scenario
            savedGMInfo = 1;
        end
        
        if ~ismember(currResultFilename,simList)
            simList = [simList; currResultFilename];
        end
    end
end

rng('shuffle')
save(simListFilepath, 'simList');

%% Data Saving/Writing
% Save the meta data information
metaDataAll = [num2cell(1:size(metaData,1))' num2cell(metaData) metaDataDescription];
metaDataHeader = {'Index' 'SourceID' 'RuptureID' 'Annualized Rate' 'Magnitude' 'Name'};
metaDataAll = [metaDataHeader; metaDataAll];
metaDataPath = [MasterFolder 'metadata.mat'];
if ~exist(metaDataPath,'file') || OverwriteFlag == 1
    save(metaDataPath,'metaDataAll');
end

% Also write the metadata to an excel file for easier reading
excelFilename = 'Metadata.xls';
excelpath = [MasterFolder excelFilename];
if ~exist(excelpath,'file') || OverwriteFlag == 1
    warning('off','MATLAB:xlswrite:AddSheet');
    xlswrite(excelpath,metaDataAll,'metadata');
end

end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Function: extractSiteList
function [siteList, nSites] = extractSiteList(siteListFilepath)

fid = fopen(siteListFilepath,'r');
siteList = textscan(fid,'%f %f %f'); % Format in: [lat long vs30]
fclose(fid);

siteList = cell2mat(siteList);

nSites = size(siteList,1);

end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Function: extractMetaData
function [metaData, metaDataDescription, numScenarios] = extractMetaData(metaDataFilepath)

fid = fopen(metaDataFilepath,'r');
metaDataCell = textscan(fid,'%d %d %f %f %s %s %s %s %s %s %s %s %s');
fclose(fid);

sourceIDkey = double(metaDataCell{1}); 
ruptureIDkey = double(metaDataCell{2}); 
annualizedRate = metaDataCell{3};
mag = metaDataCell{4};
metaData = [sourceIDkey ruptureIDkey annualizedRate mag]; % Numerical component of the meta data

% Name of source/rupture case. Note that due to white space in the name 
% inside the text file, the name is separated into different column cells.
metaDataDescription = [];
for i = 5:13
   metaDataDescription = [metaDataDescription metaDataCell{i}];
end

% Collapse the name into a single cell for each source/rupture case,
% creating a single column cell
for i = 1:length(metaDataDescription)
    for j = 2:size(metaDataDescription,2)
        metaDataDescription{i,1} = [metaDataDescription{i,1} ' ' metaDataDescription{i,j}];
    end
end

metaDataDescription = metaDataDescription(:,1);
numScenarios = size(metaData,1); % Number of earthquake scenarios

end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Function: extractIntensity
function intensity = extractIntensity(intensityFilepath, nSites)

fid = fopen(intensityFilepath,'r');
type = '';
for i = 1:(nSites*3 + 2)
    type = [type, '%f '];
end
intensity = textscan(fid, type);
fclose(fid);

intensity = cell2mat(intensity);

end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Function: getEpsilons
function epsilons = getEpsilons(Files, Input, Options, MoreInput)
% This obtains a realization of the correlated epsilons for each site
% and each earthquake scenario case
%
% INPUTS:
%
%
% epsilons : Matrix : numSimulations x n matrix indicating the
%       realizations of the correlated epsilons for each n sites for
%       each earthquake scenario (numSimulations of them)

MasterFolder = Files.MasterFolder;
RandomFieldFlag = Options.RandomFieldFlag;
SuppressMessagesFlag = Options.SuppressMessagesFlag;
OverwriteFlag = Options.OverwriteFlags(2);
numScenarios = MoreInput.numScenarios;
numSites = size(MoreInput.siteList,1);

epsilons = zeros(numScenarios,numSites);

% First, if the user desires the spectral or circulant embedding formulation, check if the site
% creator was used, which ensures at least a rectangular grided site list
% If the site creator was not found to be used, use covariance method
if (RandomFieldFlag == 2 || RandomFieldFlag == 5) && ~exist([MasterFolder 'siteInfo.mat'],'file')
    disp('Evidence of use of Site+_Creator not found, changing to covariance formulation...')
    RandomFieldFlag = 1;
end

switch RandomFieldFlag
    case 2 % Spectral
        SimInput.SpectralParams = Input.SpectralParams;
        SimInput.period = MoreInput.period;
        SimInput.MasterFolder = MasterFolder;
        SimInput.OverwriteFlag = OverwriteFlag;
        
        
        if SuppressMessagesFlag == 0
            disp('Simulating Epsilons using Spectral Formulation...')
        end

        for i = 1:numScenarios;
            
            if SuppressMessagesFlag == 0
                % Print currently simulating scenario #
                if i == 1
                    fprintf('Simulating Epsilons for Scenario %d',i)
                elseif i <= 10
                    fprintf('\b%d',i)
                elseif i <= 100
                    fprintf('\b\b%d',i)
                elseif i <= 1000
                    fprintf('\b\b\b%d',i)
                elseif i <= 10000
                    fprintf('\b\b\b\b%d',i)
                elseif i <= 100000
                    fprintf('\b\b\b\b\b%d',i)
                elseif i == 100001
                    fprintf('+.')
                elseif mod(i,4) == 1
                    fprintf('\b\b\b')
                else
                    fprintf('.')
                end
            end
            
            % Simulate the epsilons for 1 scenario
            epsilons(i,:) = Spectral_Sim(SimInput);
        end
        
        if SuppressMessagesFlag == 0
            fprintf('\n')
        end
        
        
    case 3 % Sequential
        SimInput.SequentialParams = Input.SequentialParams;
%         SimInput.MasterFolder = Files.MasterFolder;
        SimInput.period = MoreInput.period;
        SimInput.siteList = MoreInput.siteList;
        SimInput.MasterFolder = MasterFolder;
        SimInput.OverwriteFlag = OverwriteFlag;
        
        if SuppressMessagesFlag == 0
            disp('Simulating Epsilons using Sequential Formulation...')
        end

        for i = 1:numScenarios;
            
            if SuppressMessagesFlag == 0
                % Print currently simulating scenario #
                if i == 1
                    fprintf('Simulating Epsilons for Scenario %d',i)
                elseif i <= 10
                    fprintf('\b%d',i)
                elseif i <= 100
                    fprintf('\b\b%d',i)
                elseif i <= 1000
                    fprintf('\b\b\b%d',i)
                elseif i <= 10000
                    fprintf('\b\b\b\b%d',i)
                elseif i <= 100000
                    fprintf('\b\b\b\b\b%d',i)
                elseif i == 100001
                    fprintf('+.')
                elseif mod(i,4) == 1
                    fprintf('\b\b\b')
                else
                    fprintf('.')
                end
            end
            
            % Simulate the epsilons for 1 scenario
            epsilons(i,:) = Sequential_Sim(SimInput);
        end
        
        if SuppressMessagesFlag == 0
            fprintf('\n')
        end
        
    case 4 % Turning Bands
        
        SimInput.TurningBandsParams = Input.TurningBandsParams;
        SimInput.period = MoreInput.period;
        SimInput.siteList = MoreInput.siteList;
        SimInput.MasterFolder = MasterFolder;
        SimInput.OverwriteFlag = OverwriteFlag;
        
        if SuppressMessagesFlag == 0
            disp('Simulating Epsilons using Turning Bands Formulation...')
        end

        for i = 1:numScenarios;
            
            if SuppressMessagesFlag == 0
                % Print currently simulating scenario #
                if i == 1
                    fprintf('Simulating Epsilons for Scenario %d',i)
                elseif i <= 10
                    fprintf('\b%d',i)
                elseif i <= 100
                    fprintf('\b\b%d',i)
                elseif i <= 1000
                    fprintf('\b\b\b%d',i)
                elseif i <= 10000
                    fprintf('\b\b\b\b%d',i)
                elseif i <= 100000
                    fprintf('\b\b\b\b\b%d',i)
                elseif i == 100001
                    fprintf('+.')
                elseif mod(i,4) == 1
                    fprintf('\b\b\b')
                else
                    fprintf('.')
                end
            end
            
            % Simulate the epsilons for 1 scenario
            epsilons(i,:) = TurningBands_Sim(SimInput);
        end
        
        if SuppressMessagesFlag == 0
            fprintf('\n')
        end
        
    case 5 % Circulant Embedding
        
        SimInput.CircEmbParams = Input.CircEmbParams;
        SimInput.period = MoreInput.period;
        SimInput.siteList = MoreInput.siteList;
        SimInput.MasterFolder = MasterFolder;
        SimInput.OverwriteFlag = OverwriteFlag;
        
        if SuppressMessagesFlag == 0
            disp('Simulating Epsilons using Circulant Embedding Formulation...')
        end

        for i = 1:numScenarios;
            
            if SuppressMessagesFlag == 0
                % Print currently simulating scenario #
                if i == 1
                    fprintf('Simulating Epsilons for Scenario %d',i)
                elseif i <= 10
                    fprintf('\b%d',i)
                elseif i <= 100
                    fprintf('\b\b%d',i)
                elseif i <= 1000
                    fprintf('\b\b\b%d',i)
                elseif i <= 10000
                    fprintf('\b\b\b\b%d',i)
                elseif i <= 100000
                    fprintf('\b\b\b\b\b%d',i)
                elseif i == 100001
                    fprintf('+.')
                elseif mod(i,4) == 1
                    fprintf('\b\b\b')
                else
                    fprintf('.')
                end
            end
            
            % Simulate the epsilons for 1 scenario
            epsilons(i,:) = CircEmb_Sim(SimInput);
        end
        
        if SuppressMessagesFlag == 0
            fprintf('\n')
        end
        
    otherwise % Covariance
        SimInput.MasterFolder = MasterFolder;
        SimInput.OverwriteFlag = OverwriteFlag;
        SimInput.period = MoreInput.period;
        SimInput.siteList = MoreInput.siteList;
        SimInput.numScenarios = numScenarios;
        
        if SuppressMessagesFlag == 0
            disp('Simulating Epsilons using Covariance Formulation...')
        end
        
        epsilons = Covariance_Sim(SimInput);
        
end

end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [values, GroundMotionInfo, errFlag] = calculateGroundMotion(metaData,intensity,epsilons,LogFlag,MedFlag)
% This calculates the ground motion estimate using the equation:
% ln(SA) = ln(SA^<bar>) + epsilon + eta...for SA, PGA, PGV
%
% INPUTS:
%
% siteList : Matrix : n x 3 matrix listing the latitude, longitude, and
%       V_s30 values for each site for n sites. This should be in the following
%       format:
%       [latitude longitude V_s30]
% metaData : Matrix : m x 4 matrix listing the source ID, rupture ID,
%       annualized rate, and magnitude for each simulation case, of which
%       there are a total of m.
%
% intensity : Matrix : m x (n*3+2) matrix storing the sourceID, ruptureID,
%       mean, total standard deviation, and inter-event standard deviation
%       values of the ground motion for each site (for n sites) and for
%       each earthquake scenario (m cases). This should be in the following
%       format:
%       [sourceID ruptureID mu(site 1) total_std_dev(site 1) ...
%       inter-event_std_dev(site 1) ---> mu(site i) total_std_dev(site i) ...
%       inter-event_std_dev(site i)]
%
% OUTPUTS:
%
% values : Matrix : m x n matrix containing the realizations of ground
%       motion for every earthquake scenario (of which there are m) and
%       every site (of which there are n)
% GroundMotionInfo : Structure : Structure containing the ground motion
%       intensity median and standard deviation values. This contains the
%       following fields:
% GroundMotionInfo.mu : Matrix : m x n matrix containing the median values
%       for each m scenario and n site
% GroundMotionInfo.sigma : Matrix : m x n matrix containing the total
%       standard deviation for each m scenario and n site
% GroundMotionInfo.tau : Matrix : m x n matrix containing the between-event
%       standard deviation for each m scenario and n site
% GroundMotionInfo.phi : Matrix : m x n matrix containing the within-event
%       standard deviation for each m scenario and n site

errFlag = 0;
nSites = size(epsilons,2);
numSimulations = size(metaData,1);
values = zeros(numSimulations,nSites);
mus = zeros(numSimulations,nSites);
sigmas = zeros(numSimulations,nSites);
taus = zeros(numSimulations,nSites);
phis = zeros(numSimulations,nSites);

sourceIDkey = metaData(:,1); % source ID from the metaData
ruptureIDkey = metaData(:,2); % rupture ID from the metaData
sourceID = intensity(:,1); % source ID from the intensity file
ruptureID = intensity(:,2); % rupture ID from the intensity file

% Just in case the earthquake scenarios are arranged differently
% between the two files, find a mapping between the intensity file
% and the metadata file. Maps metaData -> intensity file (e.g. indices
% correspond to metaData indices, while entry values are intensity file
% indices)
rowIndMap = zeros(numSimulations,1);
for i = 1:numSimulations
    rowInd = find(sourceIDkey == sourceID(i) & ruptureIDkey == ruptureID(i));
    rowIndMap(i) = rowInd;
end

etaSite = mvnrnd(0,1,numSimulations); % between-event residual (interevent). This is constant among all sites for each simulation.

for iSite = 1:nSites
    muSite = intensity(:,(iSite-1)*3 + 3); % column vector of mean ground motion intensity
    sigmaSite = intensity(:,(iSite-1)*3 + 4); % total standard deviation
    tauSite = intensity(:,(iSite-1)*3 + 5); % standard deviation of the between-event residual (interevent)
    if tauSite(1) == -1 % the inter-event (between event) residual is not supported by the current GMPE
        errFlag = 1;
        disp('**ERROR** Missing between- and within-event residuals.')
        return
    else
        phiSite = sqrt(sigmaSite.^2 - tauSite.^2); % standard deviation of the within-event residual (intraevent)
        epsilonSite = epsilons(:, iSite); % all simulations, one site, within-event residual (intraevent)
        
        if MedFlag == 0
            GMSite = muSite + phiSite.*epsilonSite + tauSite.*etaSite ; %column vector of correlated ground motion
        else
            GMSite = muSite; %TEMP
        end
        taus(rowIndMap,iSite) = tauSite;
        phis(rowIndMap,iSite) = phiSite;
    end
    if LogFlag == 1
        GMSite = exp(GMSite);
        muSite = exp(muSite);
    end
    
    values(rowIndMap,iSite) = GMSite;
    mus(rowIndMap,iSite) = muSite;
    sigmas(rowIndMap,iSite) = sigmaSite;
    
end

GroundMotionInfo.mu = mus;
GroundMotionInfo.sigma = sigmas;
GroundMotionInfo.tau = taus;
GroundMotionInfo.phi = phis;

end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
