% Map_Generator_Master.m
% Jason Wu
% 01/26/15
%
% This function serves as a wrapper function to generate ground motion maps
% Refer to Assemble_Inputs.m for details on required inputs.

function errFlag = Map_Generator_Master(Files, Input, Options)

%% User Defined Variables

if nargin == 0

% File organization
Identifier = '022714_SF_05km';
BaseFolder = 'C:\Jason Files\Baker Research 13\';
MasterFolder = [BaseFolder 'MasterFolders\MasterFolder_' Identifier '\'];
SimulationFolder = [MasterFolder 'Simulations\'];
OpenSHAFolder = [BaseFolder 'OpenSHA\'];
OpenSHAInputFilename = ['OpenSHAInput_' Identifier '.txt'];
OpenSHAOutputFolderName = ['Output_' Identifier '\'];
siteListFilepath = [MasterFolder 'SiteList.txt'];

% Ground motion parameters

% The lists corresponding to these parameters are found in the next section
% Choose ERF from list; refer to ERFList for options
Define_ERF = 1; 
% Choose BGSeis from list; refer to BGSeisList for options
BGSeis = 2; 
% Choose floating rupture offset (recommended value is 5)
FloatingRuptureOffset = 5; 
% Choose attenuation functions from list; refer to AttenuationList for options
AttenuationChoices = [1 2 3 4];
% Choose intensity measure from list; refer to IntensityList for options
IntensityChoices = [2]; 
% Choose SA periods
SAPeriod = {'0.2'}; 
% Choose boundaries for site grid
LongBoundLeft = -122.514; 
LongBoundRight = -122.3608; 
LatBoundBot = 37.7065; 
LatBoundTop = 37.8121; 
% Choose grid spacing (meters) in latitude and longitude directions
GridSpacingLat = 500;
GridSpacingLong = 500;
% Number of batches of simulations 
nSimBatches = 5;

% Other options for calculation
OverwriteFlags = [1 1];
SuppressMessagesFlag = 0;

%% Lists
% These lists indicate the options allowed for the corresponding inputs to
% OpenSha. The  number before each line indicates the number the 
% user must assign to the corresponding input variables to choose that
% option. Lines without numbers implies that those options are not
% functional

% ERFList = {
%   1 'WGCEP (2007) UCERF2 - Single Branch';
%   2 'USGS/CGS 2002 Adj. Cal. ERF';
%   3 'WGCEP UCERF 1.0 (2005)'; 
%   4 'GEM1 CEUS ERF'};
%        
% BGSeisList = {
%   1 'Include';
%   2 'Exclude';
%   3 'Only Background'};
%           
% AttenuationList = {
%   1 'Campbell & Bozorgnia (2008)';
%   2 'Boore & Atkinson (2008)';
%   3 'Abrahamson & Silva (2008)';
%   4 'Chiou & Youngs (2008)';
%   5 'Boore & Atkinson (2006)';
%   6 'Chiou & Youngs (2006)';
%   7 'Campbell & Bozorgnia (2006)';
%     'Abrahamson & Silva (1997)';
%   8 'Boore, Joyner & Fumal (1997)';
%     'Campbell (1997) w/ erratum (2000) changes';
%     'Campbell and Bozorgnia (2003)';
%   9 'Field (2000)';
%     'Sadigh et al (1997)';
%     'SEA (Spudich et al., 1997)';
%   10 'ShakeMap (2003)';
%     'Choi & Stewart (2005)';
%     'Abrahamson (2000)';
%     'Baturay and Stewart (2003)';
%     'Bazzuro and Cornell (2004)';
%     'Goulet Et. Al. (2006)'};
% 
% IntensityList = {
%   1 'PGA';
%   2 'PGV';
%   3 'SA';
%     'MMI'};

%% Basic Input Structure Creation
Files.Identifier = Identifier;
Files.MasterFolder = MasterFolder;
Files.SimulationFolder = SimulationFolder;
Files.OpenSHAFolder = OpenSHAFolder;
Files.OpenSHAInputFilename = OpenSHAInputFilename;
Files.OpenSHAOutputFolderName = OpenSHAOutputFolderName;
Files.siteListFilepath = siteListFilepath;
Files.distanceMatFilepath = distanceMatFilepath;

Input.Define_ERF = Define_ERF; 
Input.BGSeis = BGSeis; 
Input.FloatingRuptureOffset = FloatingRuptureOffset; 
Input.AttenuationChoices = AttenuationChoices;
Input.IntensityChoices = IntensityChoices; 
Input.SAPeriod = SAPeriod; 
Gridspecs = [LongBoundLeft LongBoundRight LatBoundBot LatBoundTop GridSpacingLat GridSpacingLong];
Input.Gridspecs = Gridspecs;
Input.nSimBatches = nSimBatches;

Options.OverwriteFlags = OverwriteFlags;
Options.SuppressMessagesFlag = SuppressMessagesFlag;

else
    
%% Variable assignment if input structures are specified
MasterFolder = Files.MasterFolder;
OpenSHAInputFilename = Files.OpenSHAInputFilename;
OpenSHAOutputFolderName = Files.OpenSHAOutputFolderName;
OpenSHAFolder = Files.OpenSHAFolder;
SuppressMessagesFlag = Options.SuppressMessagesFlag;
OverwriteFlag = Options.OverwriteFlags(1);

end

%% Preparation
errFlag = 0;
if ~exist(MasterFolder,'dir')
    mkdir(MasterFolder)
end

%% Generate Site Grid

Site_Creator(Files, Input, Options)

%% Create OpenSHA input files
 
Opensha_Input_Writer(Files, Input, Options)
    
%% Run event set generator

Filename = OpenSHAInputFilename;
OutputFolder = OpenSHAOutputFolderName;
OpenSHAOutputFolder = [OpenSHAFolder OpenSHAOutputFolderName];

if ~exist(OpenSHAOutputFolder,'dir') || OverwriteFlag == 1
    [~] = rmdir(OpenSHAOutputFolder,'s');
    if SuppressMessagesFlag == 0
        disp('Running OpenSHA IM Event Set Calculator...')
    end

    origPath = cd;
    cd(Files.OpenSHAFolder);
    dos(['java -jar -Xmx500M IM_EventSetCalc_v3_0_ASCII.jar ' Filename ' ' OutputFolder]);
    cd(origPath);

end

 
%% Create simulation matrices with correlated epsilons
errFlag = Create_Simulation_Matrix(Files, Input, Options);



 