% Opensha_Input_Writer.m
% Jason Wu 
% 1/26/15
%
% This code writes the input text file required for the OpenSHA
% application. User must choose the earthquake parameters from various 
% options provided by OpenSha. This code attempts to simplify this process
% by automatically writing these choices into the text file indicated by
% the user via an index matrix.
%
% INPUTS:
%
% File Organization.
% Identifier : String : identifying name attached to all related files.
%       Usually appears as a "suffix", such as "SiteList_<Identifier>"
%       Optionally, the next 4 variables may be edited. This is just for
%       convenience.
% OpenSHAFolder : String : Indicates the path/directory of the folder
%       containing the OpenSHA application.
% InputFilename : String : Indicates the name of the text file to be written
% Masterfolder : String : Indicates the path/directory of the folder
%       containing the site list with longitude/latitude coordinates.
% siteListFilepath : String : Indicates the path to the text file containing the
%       site list with longitude/latitude coordinates.
% OutputFolder : String : Indicates the name of the output folder into
%       which the OpenSHA calculations will be saved.
%
% Ground Motion Parameters.
% Define_ERF: Scalar: Index to choose the Earthquake Rupture
%       Forecasts. Allowable indices are {1, 2, 3, 4}
% BGSeis: Scalar: Index to choose the inclusion of background seismicity in
%       the calculations. Allowable indicies are {1, 2, 3}
% FloatingRuptureOffset : Scalar : Defines the rupture offsets for floating
%       ruptures. Allowable values are between 1-100 km. 5 km is recommended
% AttenuationChoices: Vector : Vector that lists the indices corresponding
%       to the desired attenuation functions to use for the OpenSha 
%       calculations. There are up to 20 attenuation functions available.
% IntensityChoices: Vector: Vector that lists the indices corresponding to
%       the desired intensity measures to extract from OpenSHA. There are
%       up to 4 intensity measure available.
% SAPeriod: Cell : n x 1 cell containing strings of periods.
%       If 'SA' is one of the intensity measures desired, this
%       indicates the period at which SA is calculated in OpenSHA. Must be
%       in format #.# (e.g "1.0" or 1 second or "0.2" for 0.2 seconds).
%
% 
% OTHER VARIABLES:
% 
% AttenuationNum: Scalar : Indicates the number of attenuation functions to
%       be used 
% IntensityNum: Scalar : Indicates the number of types of intensity
%       measures to be reported by OpenSHA
% IntensityNumtrue: Scalar : Indicates the true number of intensity
%       measures to be reported by OpenSHA, accounting for SA's at different
%       periods
% LatLongVs30: Matrix: n x 3 matrix indicating the latitude, longitude, and 
%       V_s30 of the sites to estimate the ground motion in OpenSHA
%       **Update** V_s30 no longer included to allow OpenSHA to estimate
%       the V_s30
% NumSites: Scalar: Indicates the number of sites to estimate the ground
%       motion in OpenSHA
%
% REQUIRED PREPROCESSING:
% 
% - Assumes that a site list has already been generated named
%   'SiteList.txt' and saved in the master folder

function Opensha_Input_Writer(Files,Input,Options)

%% INPUTS (if input structures not specified)
if nargin == 0
    
% File Organization
Identifier = 'Example';

OpenSHAFolder = 'C:\Jason Files\Baker Research 13\OpenSHA\';
BaseFolder = 'C:\Jason Files\Baker Research 13\';
MasterFolder = [BaseFolder 'Masterfolder_' Identifier '\'];
InputFilename = ['OpenSHAInput_' Identifier '.txt'];
siteListPath = [MasterFolder 'SiteList.txt'];
OutputFolder = ['Output_' Identifier];

% Ground Motion Parameters
Define_ERF = 1; % Choose ERF from list; refer to ERFList for options

BGSeis = 2; % Choose BGSeis from list; refer to BGSeisList for options

FloatingRuptureOffset = 5; % Choose floating rupture offset

AttenuationChoices = [2 3]; % Choose GMPEs from list; refer to 
                        % AttenuationList for options

IntensityChoices = [1 3]; % Choose intensity measure from list; refer to 
                      % IntensityList for options

SAPeriod = {'1.0' '0.5'}; % Choose SA periods

ConstVs30 = 500; 

OverwriteFlag = 1;
SuppressMessagesFlag = 0;
DefineVs30Flag = 1;
DefineConstVs30Flag = 1;
else
%% Variable Assignment (if input structures are specified)
Identifier = Files.Identifier;

OpenSHAFolder = Files.OpenSHAFolder;
MasterFolder = Files.MasterFolder;
InputFilename = Files.OpenSHAInputFilename;
OutputFolder = Files.OpenSHAOutputFolderName;

% Ground Motion Parameters
Define_ERF = Input.Define_ERF;
BGSeis = Input.BGSeis;
FloatingRuptureOffset = Input.FloatingRuptureOffset;
AttenuationChoices = Input.AttenuationChoices;
IntensityChoices = Input.IntensityChoices;
SAPeriod = Input.SAPeriod;
ConstVs30 = Input.ConstVs30;

% Other Options
OverwriteFlag = Options.OverwriteFlags(1);
SuppressMessagesFlag = Options.SuppressMessagesFlag;
DefineVs30Flag = Options.DefineVs30Flag;
DefineConstVs30Flag = Options.DefineConstVs30Flag;

% Address of the file storing the site list (i.e. list of coordinates)
siteListPath = [MasterFolder 'SiteList.txt'];

end

%% Lists
% These lists indicate the options allowed for the corresponding inputs to
% OpenSha. The commented number after each line indicates the number the 
% user must assign to the corresponding input variables to choose that
% option. Choices that are commented out are not currently functional

ERFList = {'WGCEP (2007) UCERF2 - Single Branch'; ... % 1
           'USGS/CGS 2002 Adj. Cal. ERF'; ... % 2
           'WGCEP UCERF 1.0 (2005)'; ...% 3
           'GEM1 CEUS ERF'}; % 4
       
BGSeisList = {'Include'; ... % 1
              'Exclude'; ... % 2
              'Only Background'}; % 3
          
AttenuationList = {'Campbell & Bozorgnia (2008)'; ... % 1
                   'Boore & Atkinson (2008)'; ... % 2
                   'Abrahamson & Silva (2008)'; ... % 3
                   'Chiou & Youngs (2008)'; ... % 4
                   'Boore & Atkinson (2006)'; ... % 5
                   'Chiou & Youngs (2006)'; ... % 6
                   'Campbell & Bozorgnia (2006)'; ... % 7
%                    'Abrahamson & Silva (1997)'; ... 
                   'Boore, Joyner & Fumal (1997)'; ... % 8
%                    'Campbell (1997) w/ erratum (2000) changes'; ...
%                    'Campbell and Bozorgnia (2003)'; ... 
                   'Field (2000)'; ... % 9
%                    'Sadigh et al (1997)'; ... 
%                    'SEA (Spudich et al., 1997)'; ... 
                   'ShakeMap (2003)'};  % 10
%                    'Choi & Stewart (2005)'; ...
%                    'Abrahamson (2000)'; ... 
%                    'Baturay and Stewart (2003)'; ... 
%                    'Bazzuro and Cornell (2004)'; ... 
%                    'Goulet Et. Al. (2006)'}; 

IntensityList = {'PGA'; ... % 1
                 'PGV'; ... % 2
                 'SA'}; ... % 3
%                  'MMI'};
        
%% Calculations
% If the master folder for this project does not exist, create the folder
if exist(MasterFolder,'dir') == 0
    mkdir(MasterFolder)
end

% Address of the file storing the input to OpenSHA
inputFilePath = [OpenSHAFolder InputFilename];


% If the input file to OpenSHA already exists and we do not want to
% overwrite it, then return this function
if exist(inputFilePath,'file') && OverwriteFlag == 0
    return
end

% If there are multiples of the same choices in the input variables, just
% use the unique values
AttenuationChoices = unique(AttenuationChoices);
IntensityChoices = unique(IntensityChoices);

AttenuationNum = length(AttenuationChoices);
IntensityNum = length(IntensityChoices);

% If SA is specified, then the number of intensity measures depends on both
% the number of intensity measures specified as well as the number of
% periods specified
if isempty(find(IntensityChoices == 3, 1))
    IntensityNumtrue = length(IntensityChoices);
else
    IntensityNumtrue = length(IntensityChoices) + length(SAPeriod)-1;
end

fid = fopen(siteListPath,'r');
siteinfo = textscan(fid,'%f %f %f');
fclose(fid);
LatLongVs30 = [siteinfo{1} siteinfo{2} siteinfo{3}];

NumSites = size(LatLongVs30,1);

%% Writing 

if SuppressMessagesFlag == 0
    disp('Writing input file to OpenSHA...')
end
fid = fopen(inputFilePath,'w');

fprintf(fid, [ERFList{Define_ERF} '\n']); % Write Earthquake Rupture Forecast
fprintf(fid, [BGSeisList{BGSeis} '\n']); % Write Background Seismicity
fprintf(fid, '%d\n', FloatingRuptureOffset); % Write floating rupture offset
fprintf(fid, '%d\n', AttenuationNum); % Write number of attenuation functions
for i = 1:AttenuationNum % Write attenuation functions
    fprintf(fid, [AttenuationList{AttenuationChoices(i)} '\n']);
end
fprintf(fid, '%d\n', IntensityNumtrue); % Write number of intensity measures
for i = 1:IntensityNum % Write intensity measures
    if IntensityChoices(i) == 3 % If SA is specified, print each period
        for j = 1:length(SAPeriod)
            fprintf(fid, [IntensityList{IntensityChoices(i)} ' ' SAPeriod{j} '\n']);
        end
    else
        fprintf(fid, [IntensityList{IntensityChoices(i)} '\n']);
    end
end
fprintf(fid, '%d\n', NumSites); % Write number of sites

% If the user specified Vs30 values and does not want defined constant 
% Vs30 values, grab 3 columns from the site list data. If there is an read 
% error (no third column), then only grab 2 columns.
if DefineVs30Flag == 1 && sum(isnan(LatLongVs30(:,3))) == 0 && DefineConstVs30Flag == 0
    
    for i = 1:NumSites
        fprintf(fid, '%f %f %f\n', LatLongVs30(i,1:3));
    end
    
    % If the user specifies a constant Vs30 value, then insert that value into
    % the OpenSHA input file
elseif DefineVs30Flag == 1 && DefineConstVs30Flag == 1
    for i = 1:NumSites
        fprintf(fid, '%f %f %f\n', [LatLongVs30(i,1:2) ConstVs30]);
    end
else
    % Otherwise, only write the lat/long values and have OpenSHA genearte
    % Vs30 values using topographic slope
    for i = 1:NumSites
        fprintf(fid, '%f %f\n', LatLongVs30(i,1:2));
    end
end

fclose(fid);

if nargin == 0 % Messages are suppressed if called by the wrapper function
    disp('Now you can input the following into the command promt!')
    disp(['java -jar -Xmx500M IM_EventSetCalc_v3_0_ASCII.jar ' InputFilename ' ' OutputFolder])
end


