% Sequential_Sim.m
% Jason Wu
% 1/26/15
% Adapted from Nirmal Jayaram:
% -simulate_sa_grid_fine_observed_more_points.m
% -fine_in_coarse.m
%
% This function calculates correlated epsilons for ground motion simulation
% using Sequential Simulation techniques.
% 
% The model variogram used can be found in:
% Jayaram, N. Probabilistic Seismic Lifeline Risk Assessment Using 
% Efficient Sampling and Data Reduction Techniques. Ph. D. Dissertation, 
% Stanford University: California, 2010. http://purl.stanford.edu/hp846fv1725
% 
%
% OUTPUTS:
%
% epsilons : Matrix : n x 1 matrix indicating the
%       realizations of the correlated epsilons for each n sites for
%       each earthquake scenario (numSimulations of them)

function [epsilons, siteList] = Sequential_Sim(SimInput, coarseFlag)

% If input structures are not specified, please fill in the inputs
if nargin == 0
    %% User Input
    Identifier = 'TestRun'; 
    BaseFolder = 'C:\test\'; 
    MasterFolder = [BaseFolder 'MasterFolders\MasterFolder_' Identifier '\'];
    
    % Number of sites per row in grid
    numSitesPerRow = 25;
    % Spacing between sites (km)
    spacing = 0.5;
    % Number of sites per row in coarse grid
    coarseSize = 5;

    % Maximum distance (km) to consider screening effect when calculating conditional mean and variance
    maxDist = 30;
    % Maximum number of sites to consider screening effect when calculating conditional mean and variance
    maxScreen = 20;
    % Period of intensity measure (PGA = 0, PGV = 1, SA)
    period = 1;
    % Flag to calculate coarse grid first
    useCoarse = 0;
    
    % Flag to overwrite saved data for new simulations
    % 0 = do not overwrite
    % 1 = overwrite
    OverwriteFlag = 0;
    % Decimal degree flag
    % 1 = site list in decimal degree coordinates
    % 0 = site list in meters
    DDFlag = 0;
    
    
    %% Preparations
    xcoord = [0:(numSitesPerRow-1)]*spacing;
    ycoord = xcoord;
    [X1,Y1] = meshgrid(xcoord,ycoord);
    siteList = [Y1(:) X1(:)];
    
    coarseFlag = 0;
    
    SimInput.siteList = siteList; % Lat, Long
    SimInput.SequentialParams.useCoarse = useCoarse;
    SimInput.SequentialParams.coarseSize = coarseSize;
    SimInput.SequentialParams.maxDist = maxDist;  
    SimInput.SequentialParams.maxScreen = maxScreen;
    SimInput.period = period;
    SimInput.MasterFolder = MasterFolder;
    SimInput.OverwriteFlag = OverwriteFlag;
    
else
    MasterFolder = SimInput.MasterFolder;
    OverwriteFlag = SimInput.OverwriteFlag;
    siteList = SimInput.siteList(:,1:2); % Lat, Long
    useCoarse = SimInput.SequentialParams.useCoarse;
    coarseSize = SimInput.SequentialParams.coarseSize;
    maxDist = SimInput.SequentialParams.maxDist;  
    maxScreen = SimInput.SequentialParams.maxScreen;
    period = SimInput.period;
    
    % Decimal degree flag
    % 1 = site list in decimal degree coordinates
    % 0 = site list in meters
    DDFlag = 1;
    
    if nargin == 1
        coarseFlag = 0;
    end
    
end

dataFolder = [MasterFolder 'Data_Sequential\'];

if ~exist(dataFolder,'dir')
    mkdir(dataFolder);
end

if ~exist('counter','var')
    counter = 1; % counter for file saving purposes
end



origSiteList = siteList;

numSites = size(siteList,1);
epsilons = zeros(numSites,1); % epsilon values
epsilonsSampled = zeros(numSites,1);

% Create epsilon realizations at a coarse grid 
if coarseFlag == 0 && useCoarse == 1
    
    % Create coarse site list
    LongBoundLeft = min(siteList(:,2)); 
    LongBoundRight = max(siteList(:,2)); 
    LatBoundBot = min(siteList(:,1)); 
    LatBoundTop = max(siteList(:,1));
    xcoordcoarse = linspace(LongBoundLeft,LongBoundRight,coarseSize);
    ycoordcoarse = linspace(LatBoundBot,LatBoundTop,coarseSize);
    [X1,Y1] = meshgrid(xcoordcoarse,ycoordcoarse);
    coarseSiteList = [Y1(:) X1(:)];
    
    % Simulate epsilons on coarse site list
    SimInput.siteList = coarseSiteList;
    SimInput.numScenarios = 1;
%     [coarseEpsilons, counter] = Sequential_Sim(SimInput,1, counter);
    coarseEpsilons = Covariance_Sim(SimInput)';
    
    epsilons = [epsilons; coarseEpsilons];
    siteList = [siteList; coarseSiteList];
    epsilonsSampled = [epsilonsSampled; ones(size(coarseEpsilons))]; 
    
    % Take out the sites in the current site list that have been sampled
    % from the coarse site list
    % Function unique will by default grab the index of the last occurence
    % of a particular entry, so this will grab the indices from the coarse
    % site list over the current site list if there is an exact match in
    % coordinates
    [siteList, index] = unique(siteList,'rows');
    epsilons = epsilons(index);
    epsilonsSampled = epsilonsSampled(index);
    
else
    % Initial epsilon calculation
    epsilons(1) = randn(1);
    epsilonsSampled(1) = 1;
end

% Simulating the remaining epsilons...
for i = find(epsilonsSampled == 0)'
    
    currDataPath = [dataFolder 'data_' num2str(i) '.mat'];
    
    % For subsequent simulations, use saved previously calculated values
    if exist(currDataPath, 'file') && OverwriteFlag == 0
        epsilons(i) = getNextEpsilonEasy(currDataPath, epsilons);
    else
        % Find the indices of all sampled epsilons
        sampledInds = find(epsilonsSampled == 1);
        nearestSampledInds = [];
        distList = [];
        
        % Build a list of sampled sites that are within a certain distance of
        % the current site to be sampled
        for j = sampledInds'
            dist = distanceFromLatLong(siteList(i,:),siteList(j,:),DDFlag);
            if dist < maxDist
                % Store the distance and index of sampled site
                distList = [distList; dist];
                nearestSampledInds = [nearestSampledInds; j];
            end
        end
        
        % If there are no sites to condition on, just random normal realization
        if isempty(distList)
            epsilons(1) = randn(1);
            simpleFlag = 1;
            save(currDataPath,'simpleFlag')
        else
            % Sort by distance from sampled site to current site
            [~,sortInd] = sort(distList);
            nearestSampledInds = nearestSampledInds(sortInd);
            
            % Only take the nearest sites due to screening effect
            if length(nearestSampledInds) > maxScreen
                nearestSampledInds = nearestSampledInds(1:maxScreen);
            end
            
            sampled = [siteList(nearestSampledInds,:) epsilons(nearestSampledInds)];
            simpleFlag = 0; % Edit 070814
            save(currDataPath,'simpleFlag', 'nearestSampledInds')
            epsilons(i) = getNextEpsilon(siteList(i,:),sampled, period, currDataPath); % Edit 070814
            
        end
    end
    counter = counter+1;
    epsilonsSampled(i) = 1;
end

% Rearrange epsilon values and siteList to reflect the order of the
% original input siteList
[~,origInds] = ismember(origSiteList, siteList, 'rows');
origInds(origInds == 0) = [];
epsilons = epsilons(origInds);
siteList = siteList(origInds,:);

if coarseFlag == 1
    return
end

% Plot the realization
% epsilons = reshape(epsilons,numSitesPerRow,numSitesPerRow);
% h = imagesc(xcoord,ycoord,epsilons, [-2 2]); colormap bone
% axis square
% xlabel('Distance (km)','fontsize',24)
% ylabel('Distance (km)','fontsize',24)
% g = colorbar('Location','EastOutside');
% set(get(g,'ylabel'),'String','Epsilon','fontsize',24)
% set(gca, 'fontsize',24)


end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function epsilons = getNextEpsilon(nextCoords, sampled, period, currDataPath)
numNext = size(nextCoords, 1); % number of sites to simulate, allows for batch simulation 
sampledCoords = sampled(:,1:2);
sampledVal = sampled(:,3);
h = getDistanceMatrix([nextCoords; sampledCoords]);

if period < 1
    b = 40.7 - 15*period; % For short periods, no clustering in Vs30
%     b = 8.5 + 17.2*period; % For short periods, clustering in Vs30
else
    b = 22.0 + 3.7*period; % For long periods, period >= 1
end

% Covariance matrix for sampled sites
cov22 = exp(-3*h(numNext+1:end,numNext+1:end) / b); 

% Covariance matrix between sampled sites and sites to be sampled
cov12 = exp(-3*h(1:numNext,numNext+1:end)/b);

% Covariance matrix for sites to be sampled
cov11 = exp(-3*h(1:numNext,1:numNext)/b);

% inv_cov22 = inv (cov22); % Inverting the cov_Mat22 matrix
% cov_Mat = cov11 - (cov12) * inv_cov22 * (cov12)'; % Obtaining the conditional covariance
cov_Mat = cov11 - (cov12) * (cov22 \ (cov12)'); % Obtaining the conditional covariance
cov_Mat (cov_Mat<0) = 0; % This may not be needed

% mean_Mat = (cov12) * (inv_cov22) * sampledVal; % The conditional mean
mean_Mat = (cov12) * (cov22 \ sampledVal); % The conditional mean

[chol_Factor]=chol(cov_Mat)'; % Obtaining the upper triangular matrix using cholesky decomposition
 
uncorr_Random = randn(numNext,1); % Generating uncorrelated normal random variables
corr_Random = chol_Factor * uncorr_Random; % Correlated normally distributed epsilon values
    
epsilons = mean_Mat + corr_Random; % normally distributed RVs with given mean and std

save(currDataPath,'cov12','cov22','chol_Factor','-append')

end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
function h = getDistanceMatrix(siteList)
    % This constructs the distance matrix between the sites defined in the
    % site list.
    %
    % INPUTS:
    % 
    % siteList : Matrix : n x 2 matrix listing the latitude and longitude
    %       values for each site for n sites.
    % 
    % OUTPUTS:
    % 
    % h : Matrix : n x n matrix indicating the distance matrix for the
    %       given site list.
        nSites = size(siteList,1);
        h = zeros(nSites,nSites);
        DDFlag = 1;

    % Calculate the upper triangle of the distance matrix, as the distance
    % matrix is symmetric and has zeros on the diagonal.
        for siteI = 1:nSites
            for siteJ = (siteI+1):nSites
                LatLong1 = siteList(siteI,:);
                LatLong2 = siteList(siteJ,:);
                h(siteI,siteJ) = distanceFromLatLong(LatLong1,LatLong2,DDFlag);
            end
        end
        
    % Add in the lower triangle to form the whole distance matrix
        h = h + h';
    end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    function distance = distanceFromLatLong(LatLong1,LatLong2,DDFlag)
    % This calculates the distance in kilometers given the lat/long
    % coordiates at two sites.
    %
    % INPUTS:
    %
    % LatLong1 : Vector : 2 x 1 vector indicating the latitude/longitude
    %       coordinates for the first site (decimal degrees)
    % LatLong2 : Vector : 2 x 1 vector indicating the latitude/longiude
    %       coordinates for the second site (decimal degrees)
    %
    % OUTPUTS:
    %
    % distance : Scalar : Scalar indicating the distance between the two
    %       latitude/longitude coordinates in kilometers
    %
    % NOTE: We are currently using the Haversine formula to calculate
    
    if DDFlag ~= 1
        distance = sqrt(sum((LatLong1 - LatLong2).^2));
        return
    end
    
    radius = 6371; % Radius of the earth in km
        
    LatLong1 = LatLong1 * pi/180; % change degrees to radians
    LatLong2 = LatLong2 * pi/180; % change degrees to radians
        
    dLatLong = LatLong2 - LatLong1;
    dlat = dLatLong(1);
    dlong = dLatLong(2);
    lat1 = LatLong1(1);
    lat2 = LatLong2(1);
    
    % Change of angle using Haversine formula
    dAngle = 2*asin(sqrt(sin(dlat/2)^2+cos(lat1)*cos(lat2)*sin(dlong/2)^2));

    % Change of angle using spherical law of cosines
%     dAngle = acos(sin(lat1)*sin(lat2)+cos(lat1)*cos(lat2)*cos(dlong));

    % Calculate the separation distance between the two points
    distance = radius*dAngle;
    
    end
    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function epsilons = getNextEpsilonEasy(currDataPath, prevEpsilons)

load(currDataPath)
if simpleFlag == 1
    epsilons = randn(1);
else
    sampledVal = prevEpsilons(nearestSampledInds);
    mean_Mat = (cov12) * (cov22 \ sampledVal); % The conditional mean
    uncorr_Random = randn(1); % Generating uncorrelated normal random variables
    corr_Random = chol_Factor * uncorr_Random; % Correlated normally distributed epsilon values
    
    epsilons = mean_Mat + corr_Random; % normally distributed RVs with given mean and std
end


end